@@ -20,6 +20,7 @@ import logging as py_logging
 import os
 import time
 from typing import Any, Callable, Optional, Type, Union   # noqa: H301
+import uuid as uuid_lib
 
 from oslo_concurrency import processutils
 from oslo_log import log as logging
@@ -220,17 +221,43 @@ def convert_str(text: Union[bytes, str]) -> str:
         return text
 
 
-def get_host_nqn() -> Optional[str]:
+def get_host_nqn(system_uuid: Optional[str] = None) -> Optional[str]:
+    """Ensure that hostnqn exists, creating if necessary.
+
+    This method tries to return contents from /etc/nvme/hostnqn and if not
+    possible then creates the file calling create_hostnqn and passing provided
+    system_uuid and returns the contents of the newly created file.
+
+    Method create_hostnqn gives priority to the provided system_uuid parameter
+    for the contents of the file over other alternatives it has.
+    """
     try:
         with open('/etc/nvme/hostnqn', 'r') as f:
             host_nqn = f.read().strip()
     except IOError:
-        host_nqn = priv_nvme.create_hostnqn()
+        host_nqn = priv_nvme.create_hostnqn(system_uuid)
     except Exception:
         host_nqn = None
     return host_nqn
 
 
+def get_nvme_host_id(uuid: Optional[str]) -> Optional[str]:
+    """Get the nvme host id
+
+    If the hostid file doesn't exist create it either with the passed uuid or
+    a random one.
+    """
+    try:
+        with open('/etc/nvme/hostid', 'r') as f:
+            host_id = f.read().strip()
+    except IOError:
+        uuid = uuid or str(uuid_lib.uuid4())
+        host_id = priv_nvme.create_hostid(uuid)
+    except Exception:
+        host_id = None
+    return host_id
+
+
 def _symlink_name_from_device_path(device_path):
     """Generate symlink absolute path for encrypted devices.
 
